#    Copyright 2023 Stanford University Convex Optimization Group
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
"""
Tools to support working with bson files and strings
"""
from __future__ import annotations

from os import PathLike
from typing import Any, Dict, Union

import numpy.typing as npt
import pandas as pd

# see https://github.com/microsoft/pylance-release/issues/2019
from typing_extensions import TypeAlias

import bson
from cvx.bson.io import decode, encode

FILE = Union[str, bytes, PathLike]
MATRIX: TypeAlias = Union[npt.NDArray[Any], pd.DataFrame]
MATRICES = Dict[str, MATRIX]


def read_bson(file: FILE) -> MATRICES:
    """
    Read a bson file and prepare the bson_document needed

    Args:
        file:

    Returns:
        A dictionary of numpy arrays


    """
    with open(file, mode="rb") as openfile:
        # Reading from bson file
        return from_bson(bson_str=openfile.read())


def write_bson(file: FILE, data: MATRICES) -> int:
    """
    Write dictionary into a bson file

    Args:
        file: file
        data: dictionary of numpy arrays
    """
    with open(file=file, mode="wb") as bson_file:
        return bson_file.write(to_bson(data=data))


def to_bson(data: MATRICES) -> bytes:
    """
    Convert a dictionary of numpy arrays into a bson string

    Args:
        data: dictionary of numpy arrays
    """
    return bytes(bson.dumps({name: encode(matrix) for name, matrix in data.items()}))


def from_bson(bson_str: bytes) -> MATRICES:
    """Convert a bson string into a dictionary of numpy arrays"""
    return {name: decode(value) for name, value in bson.loads(bson_str).items()}



#    Copyright 2023 Stanford University Convex Optimization Group
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
"""
Tools to support working with json files
"""
import json
from collections.abc import Iterable
from os import PathLike
from typing import Any, Dict, Union

import numpy as np
import numpy.typing as npt
from numpyencoder import NumpyEncoder

# see https://github.com/microsoft/pylance-release/issues/2019
from typing_extensions import TypeAlias

FILE = Union[str, bytes, PathLike]
MATRIX: TypeAlias = npt.NDArray[Any]
DATA = Dict[str, Any]


def read_json(json_file: FILE) -> DATA:
    """Read a json file and return a genaerator of key-value pairs"""
    with open(json_file) as f:
        json_data = json.load(f)
        d = {}
        for name, data in json_data.items():
            if isinstance(data, Iterable):
                d[name] = np.asarray(data)
            else:
                d[name] = data

        return d


def write_json(json_file: FILE, data: DATA) -> None:
    """Write a json file with the data"""
    with open(json_file, "w") as f:
        json.dump(data, f, cls=NumpyEncoder)



#    Copyright 2023 Stanford University Convex Optimization Group
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
"""Create a dataclass from a template"""
from pathlib import Path

from jinja2 import Template


def render(names, strategy):
    # gain this dictionary through reflection
    # every table has a shortname, e.g. prices and a longer descriptive name
    # we use the shortname to name the variable refering to the code
    # names = {"A": "AAA", "B": "BBB", "C": "CCC"}
    path = Path(__file__).parent

    # open the template
    template = Template(open(path / "data_api.py.jinja2", encoding="utf-8").read())

    # we copy & paste the new class into a new file
    return template.render(names=names, strategy=strategy)
